@@ -3,6 +3,7 @@ use gcode::{GCode, Mnemonic, Span, Word};
 macro_rules! smoke_test {
     ($name:ident, $filename:expr) => {
         #[test]
+        #[cfg(feature = "std")]
         fn $name() {
             let src = include_str!(concat!(
                 env!("CARGO_MANIFEST_DIR"),
@@ -12,7 +13,7 @@ macro_rules! smoke_test {
             let src = sanitise_input(src);
 
             let _got: Vec<_> =
-                gcode::parse_with_callbacks(&src, PanicOnError).collect();
+                gcode::full_parse_with_callbacks(&src, PanicOnError).collect();
         }
     };
 }
@@ -46,7 +47,8 @@ fn expected_program_2_output() {
 
     let src = include_str!("data/program_2.gcode");
 
-    let got: Vec<_> = gcode::parse_with_callbacks(src, PanicOnError).collect();
+    let got: Vec<_> =
+        gcode::full_parse_with_callbacks(src, PanicOnError).collect();
 
     // total lines
     assert_eq!(got.len(), 20);
@@ -70,7 +72,14 @@ impl gcode::Callbacks for PanicOnError {
         panic!("Unknown content at {:?}: {}", span, text);
     }
 
-    fn gcode_buffer_overflowed(&mut self, _gcode: GCode) {
+    fn gcode_buffer_overflowed(
+        &mut self,
+        _mnemonic: Mnemonic,
+        _major_number: u32,
+        _minor_number: u32,
+        _arguments: &[Word],
+        _span: Span,
+    ) {
         panic!("Buffer overflow");
     }
 
@@ -99,6 +108,7 @@ impl gcode::Callbacks for PanicOnError {
     }
 }
 
+#[allow(dead_code)]
 fn sanitise_input(src: &str) -> String {
     let mut src = src.to_string();
     let callbacks = [handle_percent, ignore_message_lines];
@@ -110,6 +120,7 @@ fn sanitise_input(src: &str) -> String {
     src
 }
 
+#[allow(dead_code)]
 fn handle_percent(src: &str) -> String {
     let pieces: Vec<&str> = src.split('%').collect();
 
@@ -122,6 +133,7 @@ fn handle_percent(src: &str) -> String {
     }
 }
 
+#[allow(dead_code)]
 fn ignore_message_lines(src: &str) -> String {
     // "M117 Printing..." uses string arguments, not the normal char-float word
     let blacklist = ["M117"];
